package com.ruoyi.generator.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.generator.domain.Community.CommunityArticle;
import com.ruoyi.generator.domain.Community.CommunityArticleCollect;
import com.ruoyi.generator.domain.Community.CommunityArticleComment;
import com.ruoyi.generator.domain.Community.CommunityLike;
import com.ruoyi.generator.mapper.community.CommunityArticleCommentMapper;
import com.ruoyi.generator.mapper.community.CommunityArticleMapper;
import com.ruoyi.generator.mapper.community.CommunityCollectMapper;
import com.ruoyi.generator.mapper.community.CommunityLikeMapper;
import com.ruoyi.generator.vo.CommunityArticleCommentVo;
import com.ruoyi.generator.vo.CommunityArticleVo;
import com.ruoyi.system.mapper.SysUserMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * 业务 服务层实现
 *
 * @author ruoyi
 */
@Service
public class CommunityArticleServiceImpl extends ServiceImpl<CommunityArticleMapper, CommunityArticle> implements ICommunityArticleService {
    private static final Logger log = LoggerFactory.getLogger(CommunityArticleServiceImpl.class);

    @Autowired
    private CommunityArticleMapper communityArticleMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private CommunityLikeMapper communityLikeMapper;

    @Autowired
    private CommunityCollectMapper communityCollectMapper;
    @Autowired
    private CommunityArticleCommentMapper communityArticleCommentMapper;

    /**
     * 查询文章列表
     *
     * @param communityArticle 文章信息
     * @return 文章信息集合
     */
    @Override
    public List<CommunityArticleVo> selectCommunityArticleList(CommunityArticle communityArticle) {
        List<CommunityArticleVo> communityArticleVos = communityArticleMapper.selectCommunityArticleList(communityArticle);
        CommunityArticleVo communityArticleVo = null;
        Long userId = SecurityUtils.getUserId();
        for (int i = 0; i < communityArticleVos.size(); i++) {
            communityArticleVo = communityArticleVos.get(i);
            //文章下的评论
            List<CommunityArticleCommentVo> comments = communityArticleVo.getComments();
            for (CommunityArticleCommentVo communityArticleCommentVo : comments) {
                //获取评论的用户信息
                SysUser sysUser = sysUserMapper.selectUserById(communityArticleCommentVo.getUserId());
                communityArticleCommentVo.setUsername(sysUser.getUserName());
                communityArticleCommentVo.setAvatar(sysUser.getAvatar());

                //判断是否已收藏
                CommunityArticleCollect collect = communityCollectMapper
                        .selectOne(new QueryWrapper<CommunityArticleCollect>()
                                .eq("user_id", userId)
                                .eq("article_id", communityArticleCommentVo.getArticleId()));
                communityArticleVo.setCollect(!Objects.isNull(collect));

                //判断是否已点赞
                CommunityLike like = communityLikeMapper
                        .selectOne(new QueryWrapper<CommunityLike>()
                                .eq("user_id", userId)
                                .eq("article_id", communityArticleCommentVo.getArticleId()));
                communityArticleVo.setLike(!Objects.isNull(like));
            }
        }
        return communityArticleVos;
    }


    /**
     * 发送评论
     * @param communityArticleComment 评论信息
     */
    @Override
    public void sendComment(CommunityArticleComment communityArticleComment) {
        communityArticleComment.setId(null);
        communityArticleComment.setCreateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
        communityArticleComment.setUpdateBy(String.valueOf(SecurityUtils.getLoginUser().getUserId()));
        communityArticleComment.setCreateTime(DateUtils.parseDate(DateUtils.getNowDate()));
        communityArticleComment.setUpdateTime(DateUtils.parseDate(DateUtils.getNowDate()));
        communityArticleCommentMapper.insert(communityArticleComment);
    }


    /**
     * 查询该文章对应的用户是否已经点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞信息
     */
    @Override
    public CommunityLike selectCommunityLikeById(CommunityLike communityLike) {
        QueryWrapper<CommunityLike> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("article_id", communityLike.getArticleId()).eq("user_id", communityLike.getUserId());
        return communityLikeMapper.selectOne(queryWrapper);
    }


    /**
     * 新增点赞
     *
     * @param communityLike 点赞信息
     * @return 点赞是否成功
     */
    @Override
    public boolean insertCommunityLikeById(CommunityLike communityLike) {
        return communityLikeMapper.insert(communityLike) > 0;
    }


    /**
     * 删除点赞
     *
     * @param communityLike 点赞信息
     * @return 删除是否成功
     */
    @Override
    public boolean deleteCommunityLikeById(CommunityLike communityLike) {
        return communityLikeMapper.deleteById(communityLike.getId()) > 0;
    }


    /**
     * 查询该文章对应的用户是否已经收藏
     *
     * @param collect 收藏信息
     * @return 收藏信息
     */
    @Override
    public CommunityArticleCollect selectCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper
                .selectOne(new QueryWrapper<CommunityArticleCollect>()
                        .eq("article_id", collect.getArticleId())
                        .eq("user_id", SecurityUtils.getUserId()));
    }

    /**
     * 新增收藏
     *
     * @param collect 收藏信息
     * @return 收藏是否成功
     */
    @Override
    public boolean insertCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper.insert(collect) > 0;
    }

    /**
     * 删除收藏
     *
     * @param collect 收藏信息
     * @return 删除是否成功
     */
    @Override
    public boolean deleteCommunityCollectById(CommunityArticleCollect collect) {
        return communityCollectMapper.deleteById(collect) > 0;
    }
}